#-----------------------------------------------------------
#	A_law Encoder and Decoder
#-----------------------------------------------------------
#  * Function
#    compress 13 bit linear data to 8 bit A_law data
#    1. linear scale section
#                 A
#	y(x) = --------- sgn(x)    0=< |x| < xmax/A, A=87.56
#		1 + lnA         ,
#    2. log scale section
#            1+ln(A|x|/xmax)
#	xmax --------------- sgn(x)   xmax/A < |x| < xmax, A=87.56
#	         1+lnA             ,
#  * Assumption and Constraint
#
#  * Input Value
#	RAM0[10] : input 13 bit linear data
#  * Temporary Values
#	RAM0[00] : segment code
#	RAM0[01] : segment base value
#	RAM0[11] : shift value to get interval number(0~15)
#	RAM1[00] : sign
#  * Output Value
#	RAM0[10] : encoder output 8 bit
#  * Cycles
#    encoder : 33 cycles, decoder : 43 cycles
#-------------------------------------------------------------------
org	0000
dw      0004
dw      0004
dw      0004
dw      0004
org	0004
    ## A_law encoder
    # Input 13 bit linear data
    	ld	a, (r3|10)	# 16 bit linear data
    	mod	always, shr
    	mod	always, shr
    	mod	always, shr	# 13 bit linear data
    	cmpi	00
    	ldi	(r7|00), 0080	# sign bit = 1 when pos
    	bra	n=0, @pos_operat
    neg_operat:
    	mod	always, abs
        ldi     (r7|00), 0000   # sign bit = 0 when neg
    pos_operat:
    	cmpi	a, 200		# cmp with 512
	bra	n=1, @lower_than_512
	cmpi	a, 400
	bra	n=1, @fix_seg_5	# cmp with 1024
	cmpi	a, 800
	bra	n=1, @fix_seg_6
	ldi	(r3|00), 70	# fix segment 7
	ldi	(r3|01), 800
	ldi	(r3|11), 0100
	bra	always, @fix_seg_end
    fix_seg_5:
        ldi     (r3|00), 50     # fix segment 5
	ldi	(r3|01), 200
	ldi	(r3|11), 0400
	bra	always, @fix_seg_end
    fix_seg_6:
        ldi     (r3|00), 60    # fix segment 6
	ldi	(r3|01), 400
	ldi	(r3|11), 0200
	bra	always, @fix_seg_end
    lower_than_512:
     	cmpi	80
     	bra	n=1, @lower_than_80
     	cmpi	100
     	bra	n=1, @fix_seg_3
     	ldi	(r3|00), 40	# fix segment 4
	ldi	(r3|01), 100
	ldi	(r3|11), 0800
	bra	always, @fix_seg_end
    fix_seg_3:
        ldi     (r3|00), 30     # fix segment 3
	ldi	(r3|01), 80
	ldi	(r3|11), 1000
	bra	always, @fix_seg_end
    lower_than_80:
        cmpi	40
        bra	n=1, @fix_seg_1
        ldi     (r3|00), 20     # fix segment 2
	ldi	(r3|01), 40
	ldi	(r3|11), 2000
	bra	always, @fix_seg_end
    fix_seg_1:
        ldi     (r3|00), 10     # fix segment 1
	ldi	(r3|01), 00
	ldi	(r3|11), 4000
    fix_seg_end:
        sub 	a, (r3|01)	# sub segment start value
	ld	x, a
	ld	y, (r3|11)	# shift value
	ld	a, p
	or	a, (r7|00)	# combine sign
	or 	a, (r3|00)	# combine segment & intervals
	ld	(r3|10), a  	# save A_law 8 bit code
    	
    ## A_law decoder
    # input A_law code
    	ld	a, (r3|10)	# load A_law 8 bit code
    	andi	a, 0070		# get segment
    	cmpi	40
    	bra	n=1, @lower_than_seg4
    	cmpi	60
    	bra	n=1, @lower_than_seg6
    	cmpi	70
    	bra	n=1, @fix_base_400
    	ldi	(r3|01), 800       	# seg=7
    	ldi	(r3|11), 80		# interval size=80
    	bra	always, @fix_base_end
    fix_base_400:
    	ldi	(r3|01), 400       	# seg=6
    	ldi	(r3|11), 40		# interval size=40
    	bra	always, @fix_base_end
    lower_than_seg6:
    	cmpi	50
    	bra	n=1, @fix_base_100
    	ldi	(r3|01), 200       	# seg=5
    	ldi	(r3|11), 20		# interval size=20
    	bra	always, @fix_base_end
    fix_base_100:
    	ldi	(r3|01), 100       	# seg=4
    	ldi	(r3|11), 10		# interval size=10
    	bra	always, @fix_base_end
    lower_than_seg4:
	cmpi	20
	bra	n=1, @fix_base_0
	cmpi	30
	bra	n=1, @fix_base_40
    	ldi	(r3|01), 80    		# seg=3
    	ldi	(r3|11), 8		# interval size=8
    	bra	always, @fix_base_end
    fix_base_40:
    	ldi	(r3|01), 40    		# seg=2
    	ldi	(r3|11), 4		# interval size=4
    	bra	always, @fix_base_end
    fix_base_0:
    	ldi	(r3|01), 00    		# seg=1
    	ldi	(r3|11), 2		# interval size=2
    	bra	always, @fix_base_end
    fix_base_end:
        ld	a, (r3|10)      # 8 bit A_law code
        andi	0f		# get interval numbers
        ld      x, a            # for interval number * interval size
        bra	z=0, @non_zero_interval
    non_zero_interval:
        ld	y, (r3|11)	# interval size
        ld	a, p            # interval number * interval size
        mod	always, shr	# compensate double in mac oper.
        ld	stack, ext7
        ld	a, (r3|01)
        add	a, stack  	# x(n-1)= seg. base value + interval value
        ld	stack, a        # save x(n-1) to stack for y(n)
        add	a, (r3|11)      # get x(n)
	add	a, stack
	mod	always, shr     # y(n) = ( x(n) + x(n-1) ) / 2
	ld	stack, a	# save y(n) to stack for sign consideration
	ld	a, (r3|10)
	andi	a, 80           # get sign info.  a=80  -> sign=pos
	ld	a, stack
	bra	z=0, @pos_sign
	mod	always, neg     # if sign=neg  then y(n)=-y(n)
    pos_sign:
    	ld	(r3|10), a	# save y(n)
    decoder_end:
    	bra	always, @decoder_end
	
    	
        
     
