#---------------------------------------------------------------#
#	TITLE : IIR 4th filter(Canonic)				#
#	Ver : 0.0                                               #
#	Date : AUG. 3. 1995                                     #
#	Author : SunKyu-Kim                                     #
#	Program : IIR_4.sc                                      #
#---------------------------------------------------------------#
#  * Function :                                                 #
#                                                               #
#        w(n) = x(n) - ai1*w(n-1) - ai2*w(n-2)                  #
#        y(n) = w(n) + bi1*w(n-1) + bi2*w(n-2)                  #
#                                                               #
#                                                               #
# 	                         ( bi0 = 1 )                    #
#	x(n) ----(-)---->---- w(n) ---->----(+)---- y(n)        #
#       	  |            |             |                  #
#       	  |            |             |                  #
#       	  |           z-1            |                  #
#       	  |            |             |                  #
#                 |    ai1     |      bi1    |                  #
#                 |-----<--- w(n-1) --->-----|                  #
#                 |            |             |                  #
#                 |            |             |                  #
#                 |           z-1            |                  #
#                 |            |             |                  #
#                 |    ai2     |      bi2    |                  #
#                 ------<--- w(n-2) --->------                  #
#                                                               #
#  * Input values						#
#	RAM1[10] : x[n]                                         #
#  * Parameters                                                 #
#	RAM1[20~2f] : w[n]                                      #
#  * Coefficients                                               #
#	RAM0[20~2F] : ak1, ak2, bk1, bk2, k=1~4                 #
#  * Output values                                              #
#	RAM1[11~ ] : y[n]                                       #
#                                                               #
#      RAM Bank 0                  RAM Bank 1                   #
#    ---------------            -----------------               #
#    |             |            |               |               #
#    |-------------|            |---------------|               #
#    |             |    r4 = 10 |     x[n]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[1]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[2]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[3]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[4]      |  <= output    #
#    |-------------|            |---------------|               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |-------------|            |---------------| <- circular   #
#    |             |    r5 = 20 |    w1[n-2]    |    start      #
#    |-------------|            |---------------|               #
#    |             |         21 |    w1[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         22 |     w1[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         23 |    w2[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         24 |    w2[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         25 |     w2[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         26 |    w3[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         27 |    w3[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         28 |     w3[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         29 |    w4[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         2a |    w4[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         2b |     w4[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         2c |               |               #
#    |-------------|            |---------------|               #
#    |             |         2d |               |               #
#    |             |            |---------------|               #
#    |             |         2e |               |               #
#    |-------------|            |---------------|               #
#    |             |         2f |               |               #
#    ---------------            ----------------- <- circular   #
#                                                      end      #
#  * Cycle numbers                                              #
#	maximum 52 cycles					#
#								#
#***************************************************************#

        org     0000        # Interrupt vector table for SSP1601M

	dw   	0004   # Reset address  #
	dw   	0004   # INT0 address   #
   	dw   	0004   # INT1 address   #
   	dw   	0004   # INT2 address   #
   
   	org  	0004                    # for SSP1601S

#=================<< test vector -- simlulation data >>==================#
    initial_processing():
        ldi	st, 0
   	mod  	f, setop
	ldi	r0, 20
	ldi	r5, 20
	ldi	st, 0304	# loop size : 16
	ldi	(r7|00), @table

#-----------------------------------------------------------------#
#                      initialize r5 value                        #
#-----------------------------------------------------------------#
	ld	a, r5
	ld	002, a		# for w[n-2], w[n-1], w[n]


#-----------------------------------------------------------------#
#                      load_coefficients( )                       #
#-----------------------------------------------------------------#
	ldi	a, 10		# coeff : ak2, ak1, bk2, bk1, k=1,2,3,4
     loop:
	ld	x, ((r7|00))
	ld	(r0+!), x
	subi	01
	bra	z=0, @loop

#-----------------------------------------------------------------#
#                      load_input_data()                          #
#-----------------------------------------------------------------#
     #- loading speech_data from program memory, only for simulation  -#
	ldi	r2, 01
	ldi  	a, @SIM_DATA
	ld	(r2), a

     Input_Data:
	andi	00
	ldi	r2, 01		# Save address of "@SIM_DATA"
	ldi	r4, 10

     # LOADing :
	ld	a, (r2)
	ld   	stack, (a)
	ld   	(r4), stack
	addi 	01              # increase program memory addr.
	ld   	(r2), a
##################################
#        IIR Filter (x4)
##################################
	ldi	r0, 20
	ldi	r4, 10
	ld	a, 002		# save r5, circular buffer pointer
	ld	r5, a
#---------------------------------
 	call	always, @Canonic4
	call	always, @Canonic4
	call	always, @Canonic4
	call	always, @Canonic4
	ld	-, (r5+)
	ld	-, (r5+)
	ld	-, (r5+)
	ld	-, (r5+)
	ld	-, (r5+)
#---------------------------------
	ld	a, r5
	ld	002, a
##################################
	bra	always, @Input_Data

table:
dw   15a2
dw   00cc
dw   cc61
dw   0000
dw   2a95
dw   86ee
dw   d0d9
dw   0000
dw   4e3f
dw   3bde
dw   b6c5
dw   0000
dw   3021
dw   a336
dw   cf70
dw   0000

#dw     2c51     # D3AF     # section  1 coefficient A2        -11345
#dw     abd8	# 5428     # section  1 coefficient A1         21544
#dw     9A88     # section  1 coefficient B2        -25976
#dw     0000     # section  1 coefficient B1             0
#dw     2c51     # D3AF     # section  2 coefficient A2        -11345
#dw     5428	# ABD8     # section  2 coefficient A1        -21544
#dw     EA24     # section  2 coefficient B2         -5596
#dw     0000     # section  2 coefficient B1             0
#dw     32cf	# CD31     # section  3 coefficient A2        -13007
#dw     a9b1	# 564F     # section  3 coefficient A1         22095
#dw     84D9     # section  3 coefficient B2        -31527
#dw     0000     # section  3 coefficient B1             0
#dw     32cf     # CD31     # section  4 coefficient A2        -13007
#dw     564f	# A9B1     # section  4 coefficient A1        -22095
#dw     D336     # section  4 coefficient B2        -11466
#dw     0000     # section  4 coefficient B1             0

    Canonic4:
#        w(n) = x(n) - a1(n)*w(n-1) - a2(n)*w(n-2)
#        y(n) = w(n) + b1(n)*w(n-1) + b2(n)*w(n-2)
	mld	(r5+), (r0+!)	# acc <- 0, p <- a2(n)*w(n-2)
	mpya	(r5+), (r0+!)	# acc <- acc + p
				# p   <- a1(n)*w(n-1)
	mpya    (r5), (r0)      # acc <- a1(n)*w(n-1) + a2(n)*w(n-2)
				# p : don't care
	add     a, (r4+)        # acc <- x(n) + a1(n)*w(n-1) + a2(n)*w(n-2)
		# a1 and a2 coefficient is changed as the mark of opposite.
		# so, "Sub" instruction is changed as "Add' instruction.
	ld	(r5-), a	# saving for next calculation
	ld	-, (r5-)	# go (r5) to w(n-2) for next calculation
	mld     (r5+), (r0+!)   # acc <- 0, p <- b2(n)*w(n-2)
	#mpya    (r5+), (r0+!)  # acc <- 0, p <- b2(n)*w(n-2)
	mpya    (r5+), (r0+!)   # acc <- acc + p
                                # p   <- b1(n)*w(n-1)
	mpya    (r5), (r0)      # acc <- b1(n)*w(n-1) + b2(n)*w(n-2)
	add     a, (r5+)        # acc <- w(n) + b1(n)*w(n-1) + b2(n)*w(n-2)
	ld      (r4), a         # saving for next input
				# last saving is for output.
	ld	pc, stack

     SIM_DATA:
include iir5b_in.hex

