#---------------------------------------------------------------#
#	TITLE : IIR 5th filter(Canonic)                         #
#	Ver : 0.0                                               #
#	Date : JUL. 14. 1995                                    #
#	Author : SunKyu-Kim                                     #
#	Program : IIR_5.sc                                      #
#---------------------------------------------------------------#
#  * Function :                                                 #
#                                                               #
#        w(n) = x(n) - ai1*w(n-1) - ai2*w(n-2)                  #
#        y(n) = bi0*w(n) + bi1*w(n-1) + bi2*w(n-2)              #
#                                                               #
#                                                               #
#                              bi0                              #
#	x(n) ----(-)---->---- w(n) ---->----(+)---- y(n)        #
#       	  |            |             |                  #
#       	  |            |             |                  #
#       	  |           z-1            |                  #
#       	  |            |             |                  #
#                 |    ai1     |      bi1    |                  #
#                 |-----<--- w(n-1) --->-----|                  #
#                 |            |             |                  #
#                 |            |             |                  #
#                 |           z-1            |                  #
#                 |            |             |                  #
#                 |    ai2     |      bi2    |                  #
#                 ------<--- w(n-2) --->------                  #
#                                                               #
#  * Input values						#
#	RAM1[10] : x[n]                                         #
#  * Parameters                                                 #
#	RAM1[20~33] : w[n]                                      #
#  * Coefficients                                               #
#	RAM0[20~33] : ak1, ak2, bk0, bk1, bk2, k=1~4            #
#  * Output values                                              #
#	RAM1[11~ ] : y[n]                                       #
#                                                               #
#      RAM Bank 0                  RAM Bank 1                   #
#    ---------------            -----------------               #
#    |             |            |               |               #
#    |-------------|            |---------------|               #
#    |             |    r4 = 10 |     x[n]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[1]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[2]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[3]      |               #
#    |-------------|            |---------------|               #
#    |             |            |     y[4]      |  <= output    #
#    |-------------|            |---------------|               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |             |            |               |               #
#    |-------------|            |---------------| <- circular   #
#    |             |    r5 = 20 |    w1[n-2]    |    start      #
#    |-------------|            |---------------|               #
#    |             |         21 |    w1[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         22 |     w1[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         23 |    w2[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         24 |    w2[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         25 |     w2[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         26 |    w3[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         27 |    w3[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         28 |     w3[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         29 |    w4[n-2]    |               #
#    |-------------|            |---------------|               #
#    |             |         2a |    w4[n-1]    |               #
#    |-------------|            |---------------|               #
#    |             |         2b |     w4[n]     |               #
#    |-------------|            |---------------|               #
#    |             |         2c |               |               #
#    |-------------|            |---------------|               #
#    |             |         2d |               |               #
#    |             |            |---------------|               #
#    |             |         2e |               |               #
#    |-------------|            |---------------|               #
#    |             |         2f |               |               #
#    ---------------            ----------------- <- circular   #
#                                                      end      #
#  * Cycle numbers                                              #
#       maximum 52 cycles	                                #
#***************************************************************#

        org     0000                    # Interrupt vector table for SSP1601M
	                             #
	dw   	0004   # Reset address  #
	dw   	0004   # INT0 address   #
   	dw   	0004   # INT1 address   #
   	dw   	0004   # INT2 address   #
   
   	org  	0004                    # for SSP1601S

#=================<< test vector -- simlulation data >>==================#

    Initialize_Processor():
         ld      st, a
         mod     f, setop

         ldi     r0, 20
         ldi     r5, 20
         ldi     st, 0304
         ldi     (r7|00), @table

#-----------------------------------------------------------------#
#                      initialize r5 value                        #
#-----------------------------------------------------------------#
         ld      a, r5
         ld      002, a

#-----------------------------------------------------------------#
#                      load_coefficients( )                       #
#-----------------------------------------------------------------#
         ldi     a, 14
    loop:
         ld      x, ((r7|00))
         ld      (r0+!), x
         subi    01
         bra     z=0, @loop

#-----------------------------------------------------------------#
#                      load_input_data()                          #
#-----------------------------------------------------------------#
#- loading speech_data from program memory, only for simulation  -#
         ldi     r2, 01
         ldi     a, @SIM_DATA
         ld      (r2), a

    Input_Data:
         andi    00
         ldi     r2, 01                  # Save address of "@SIM_DATA"
         ldi     r4, 10

    #LOADing :
         ld      a, (r2)
         ld      stack, (a)
         ld      (r4), stack
         addi    01
         ld      (r2), a
##################################
         ldi     r0, 20
         ldi     r4, 10
         ld      a, 002
         ld      r5, a
#---------------------------------
         call    always, @Canonic5
         call    always, @Canonic5
         call    always, @Canonic5
         call    always, @Canonic5
         ld      -, (r5+)
         ld      -, (r5+)
         ld      -, (r5+)
         ld      -, (r5+)
         ld      -, (r5+)
#---------------------------------
         ld      a, r5
         ld      002, a
##################################
         bra     always, @Input_Data

    table:
         dw     15a2     #   ea5e        section  1 coefficient A2
         dw     00cc     #   ff34        section  1 coefficient A1
         dw     CC61     #   section  1 coefficient B2
         dw     0000     #   section  1 coefficient B1
         dw     339F     #   section  1 coefficient B0
         dw     2a95     #   D56B        section  2 coefficient A2
         dw     86ee     #   7912        section  2 coefficient A1
         dw     D0D9     #   section  2 coefficient B2
         dw     0000     #   section  2 coefficient B1
         dw     2F27     #   section  2 coefficient B0
         dw     4e3f     #   B1C1        section  3 coefficient A2
         dw     3bde     #   C422        section  3 coefficient A1
         dw     B6C5     #   section  3 coefficient B2
         dw     0000     #   section  3 coefficient B1
         dw     493B     #   section  3 coefficient B0
         dw     3021     #   CFDF        section  4 coefficient A2
         dw     a336     #   5CCA        section  4 coefficient A1
         dw     CF70     #   section  4 coefficient B2
         dw     0000     #   section  4 coefficient B1
         dw     3090     #   section  4 coefficient B0

    Canonic5:
         mld     (r5+), (r0+!)
         mpya    (r5+), (r0+!)
         mpya    (r5), (r0)
         #add    a, p
         #mod    always, shr
         #ld     x, a
         #ld     a, (r4+)
         #sub    a, x
         add     a, (r4+)
         ld      (r5-), a
         ld      -, (r5-)

#        w(n) = x(n) - ai1*w(n-1) - ai2*w(n-2)
#        y(n) = bi0*w(n) + bi1*w(n-1) + bi2*w(n-2)

         mld     (r5+), (r0+!)
         mpya    (r5+), (r0+!)
         mpya    (r5+), (r0+!)
         mpya    (r5), (r0)
         #add    a, p
         #mod    always, shr
         ld      (r4), a
         ld      pc, stack

    SIM_DATA:
include iir5b_in.hex

         # for comparing output data
         # include out.hex

         org     fffc                     # for SSP1601S
         dw      0004  # Reset address    #
         dw      0004  # INT0 address     #
         dw      0004  # INT1 address     #
         dw      0004  # INT2 address     #

