#---------------------------------------------------------------------------
#	TITLE : IIR Transepose Filter
#	Ver : 0.0
#	Date : JUL. 28. 1995
#	Author : SunKyu-Kim
#	Program : Transpose.sc
#---------------------------------------------------------------------------
		
#		y(n) = v[n-1] + x[n]*b01
#		v[n] = x[n]*b11 + a11*y[n] + v[n-1]
#		v[n] = x[n]*b21 + a21*y[n]


#                                     bi0
#       x(n) ----------->---- v(n) ---->----------- y(n)
#                 |            |             |
#                 |            |             |
#                 |           z-1            |
#                 |            |             |
#                 |    ai1     |      bi1    |
#                 |----->--- v(n-1) ---<-----|
#                 |            |             |
#                 |            |             |
#                 |           z-1            |
#                 |	       |	     |
#                 |    ai2     |      bi2    |
#                 ------>--- v(n-2) ---<------
#
#  Computation time :
#	maxumum 60 cycles
#***********************************************************************/

	org  	0000                    # Interrupt vector table for SSP1601M
	                             #
	dw   	0004   # Reset address  #
	dw   	0004   # INT0 address   #
   	dw   	0004   # INT1 address   #
   	dw   	0004   # INT2 address   #
   
   	org  	0004                    # for SSP1601S

#=================<< test vector -- simlulation data >>==================#

Initialize_Processor():
	andi 	00
   	ld   	st, a
   	mod  	f, setop
   	ld   	x,  a
   	ld   	r0, x
   	ld   	r4, x
   	ld   	r1, x

RAM0_Ini:          # initialize RAM0 bank
	ld   	(r0+!), x
   	ld   	a, r0
   	cmpi 	00
   	bra  	z=0, @RAM0_Ini
  
RAM1_Ini:          # initialize RAM1 bank
   	ld   	(r4+!), x
   	ld   	a, r4
   	cmpi 	00
   	bra  	z=0, @RAM1_Ini
 
#ldi	r6, 50		# output_data

ldi	r0, 20
ldi	r5, 20
ldi	st, 0d04
ldi	(r7|00), @table

#-----------------------------------------------------------------#
#                      load_coefficients( )                       #
#-----------------------------------------------------------------#
ldi	a, 14
loop:
ld	x, ((r7|00))
ld	(r0+!), x
subi	01
bra	z=0, @loop

#-----------------------------------------------------------------#
#                      load_input_data()                          #
#-----------------------------------------------------------------#
#- loading speech_data from program memory, only for simulation  -#
ldi	r2, 01
ldi  	a, @SIM_DATA
ld	(r2), a

Input_Data:
andi	00
ldi	r2, 01			# Save address of "@SIM_DATA"
ldi	r4, 10

#LOADing :
ld	a, (r2)
ld   	stack, (a)
ld   	(r4), stack
addi 	01
ld   	(r2), a
##################################
ldi	r0, 20
#ldi	r4, 10
ldi	r5, 20
#---------------------------------
call	always, @Transpose
call	always, @Transpose
call	always, @Transpose
call	always, @Transpose
##################################
ld 	-, -
bra	always, @Input_Data

table:
dw    1479      #   5241      /* section  1 coefficient B0 
dw    0000      #      0      /* section  1 coefficient B1 
dw    B5CD      # -18995      /* section  1 coefficient A1 
dw    E8F3      #  -5901      /* section  1 coefficient A2
dw    EB87      #  -5241      /* section  1 coefficient B2 
dw    57B5      #  22453      /* section  2 coefficient B0 
dw    0000      #      0      /* section  2 coefficient B1 
dw    4A33      #  18995      /* section  2 coefficient A1 
dw    E8F3      #  -5901      /* section  2 coefficient A2 
dw    A84B      # -22453      /* section  2 coefficient B2 
dw    0F4F      #   3919      /* section  3 coefficient B0 
dw    0000      #      0      /* section  3 coefficient B1 
dw    6083      #  24707      /* section  3 coefficient A1 
dw    D205      # -11771      /* section  3 coefficient A2 
dw    F0B1      #  -3919      /* section  3 coefficient B2 
dw    4EB5      #  20149      /* section  4 coefficient B0 
dw    0000      #      0      /* section  4 coefficient B1 
dw    CFBF      # -12353      /* section  4 coefficient A1 
dw    E903      #  -5885      /* section  4 coefficient A2 
dw    B14B      # -20149      /* section  4 coefficient B2 


Transpose:
mld     (r4), (r0+!)             #  p <- b1*x[n]
ld      a, (r5+)                #  v[n-1]
mpya    (r4+!), (r0+!)            #  p <- b1*x[n],  a <- v[n-1]
ld      (r4), a                 #  y <- a
and     a, -                    #  for deleting low 16bit -> 0000h (in low)
ld      a, (r5-)                #  a <- v[n-2]
mpya    (r4), (r0+!)             #  p <- a1*y[n],  a <- v[n-2] + b1*x[n]
mpya    (r4), (r0)              #  a <- v[n-1] = a1*y[n] + v[n-2] + b1*x[n]
#add    a, p                    #
#mod    always, shr             #
ld      (r5+), a                #  v[n-1] <- a  : 2nd is completed.
mld     (r4-), (r0+!)            #  p <- a2*y[n]
mpya    (r4+!), (r0+!)            #  p <- b2*x[n]
mpya    (r4), (r0)              #  a <- a2*y[n] + b2*x[n]
#add    a,p                     #
#mod    always, shr             #
ld      (r5+), a                #  a <- v[n-2] = a2*y[n] + b2*x[n]
ld      pc, stack               #
                                
SIM_DATA:                       
include iir5b_in.hex

# for comparing output data
# include out.hex
   	
org   	fffc                     # for SSP1601S
dw    	0004  # Reset address    #
dw    	0004  # INT0 address     #
dw    	0004  # INT1 address     #
dw    	0004  # INT2 address     #
