#---------------------------------------------------------------#
#	TITLE : All pole lattice filter(Canonic)		#
#	Ver : 0.0                                               #
#	Date : SEP. 3. 1996                                     #
#	Author : HeeSu-Kim                                      #
#	Program : latt_p.sc                                     #
#---------------------------------------------------------------#
#  * Function :                                                 #
#        x(n) = fN(n)			                        #
#        fm-1(n) = fm(n) - km*gm-1(n-1)		                #
#        gm(n) = km*fm-1(n) + gm-1(m-1)                         #
#        y(n) = f0(n) = g0(n)					#
#                                                               #
#  x(n)=fN(n)                                        f0(n)=y(n) #
#   ---->(+)------------------ - - ---(+)--------------------   #
#             \   /                         \   /           |   #
#          -kN  \  kN                    -k1  \  k1         |   #
#             /   \                         /   \           |   #
#   <----(+)---------->(z-1)-- - - ---(+)----------(z-1)-----   #
#        gN(n)                                		 g0(n)  #
#                                                               #
#  * Input values						#
#	RAM1[10] : x[n]                                         #
#  * Parameters                                                 #
#	RAM1[20~2f] : g[n], f[n]                                #
#  * Coefficients                                               #
#	RAM0[20~2F] : k1, k2, ... , kN                          #
#  * Output values                                              #
#	RAM1[11~ ] : fN[n]                                      #
#                                                               #
#  * Cycle numbers                                              #
#      10-tap filter : maximum 142 cycles			#
#								#
#***************************************************************#

        org     0000        # Interrupt vector table for SSP1601M

	dw   	0004   # Reset address  #
	dw   	0004   # INT0 address   #
   	dw   	0004   # INT1 address   #
   	dw   	0004   # INT2 address   #
   
   	org  	0004                    # for SSP1601S
    # load reflection parameter k
        ldi	(r3|00), @k_table
    # !! The order of k must be from m to 0
        ldi     r0, 10
        ldi	a, 0a
    k_load_loop:
        ld      x, ((r3|00))
        ld      (r0+), x
        subi	1
        bra	n=0, @k_load_loop
    # get input x(n)
        ldi	(r3|00), @input_dat
        ld	a, st
        ori	0200
        ld	st, a
        mod	f, setop
        ldi	r4, 10 		# x[n]  or  fm-1[n]  pointer
    	ldi	r2, 40
        ldi	(r3|01), be
    latt_p_routine:
        ld      a, ((r3|00))
        mod	always, shr
        mod	always, shr
        ld	(r4), a
    ## lattice filtering
    	call	always, @latt_p_main()
    	ld	a, (r3|01)
    	subi	1
    	ld	(r3|01), a
    	bra	n=0, @latt_p_routine
    end_idle:
    	bra	always, @end_idle
        
    latt_p_main():
    	ldi	r0, 10		# k pointern - start km
        ldi	r5, 12		# g[n-1] pointer - start gm-1[n-1]
    # get f9[n] = x[n]
    # get f0[n] ~ f8[n],  g1[n] ~ g9[n]
        call	always, @cal_f_g   # f8[n], g9[n]
        call    always, @cal_f_g   # f7[n], g8[n]
        call    always, @cal_f_g   # f6[n], g7[n]
        call    always, @cal_f_g   # f5[n], g6[n]
        call    always, @cal_f_g   # f4[n], g5[n]
        call    always, @cal_f_g   # f3[n], g4[n]
        call    always, @cal_f_g   # f2[n], g3[n]
        call    always, @cal_f_g   # f1[n], g2[n]
        call    always, @cal_f_g   # f0[n], g1[n]
    # get g0[n]
        ld	a, (r4)		# f0[n]=g[n]=y[n]
    	ld	-, (r5-)
        ld	(r5), a		# save g0[n]
        ld	(r2+), a	# save y[n]
    lattice_p_end:
        ret
        
    cal_f_g:
	mld	(r5), (r0)	# km*gm-1[n-1]
	ld  	a, (r4)		# fm[n]
        mpys    (r5), (r0)      # fm-1[n] = fm[n-1] - km*gm-1[n-1]
	ld	(r4), a    	# save fm-1[n]
        mld     (r4), (r0+)     # km*fm-1[n]
        ld      a, (r5-)        # gm-1[n-1]
        mpya    (r4), (r0)      # gm[n] = km*fm-1[n] + gm-1[n-1]
	ld	(r5+), a  	# save gm[n]
	ld	-, (r5+) 	# goto gm-2[n]
	ret
    k_table:
dw	3332
dw	d99a
dw	4ccc
dw	0ccc
    
dw	ccce
dw	0ccc
dw	c001
dw      2666
dw	1999
dw	ccce
dw	f334
dw	2666
dw	b334
dw 	0ccc
    

    input_dat:
dw      0000
dw      0809
dw      100a
dw      17fb
dw      1fd4
dw      278d
dw      2f1e
dw      367f
dw      3da9
dw      4495
dw      4b3b
dw      5196
dw      579e
dw      5d4e
dw      629f
dw      678d
dw      6c12
dw      7029
dw      73d0
dw      7701
dw      79bb
dw      7bf9
dw      7dba
dw      7efc
dw      7fbe
dw      7fff
dw      7fbe
dw      7efc
dw      7dba
dw      7bf9
dw      79bb
dw      7701
dw      73d0
dw      7029
dw      6c12
dw      678d
dw      629f
dw      5d4e
dw      579e
dw      5196
dw      4b3b
dw      4495
dw      3da9
dw      367f
dw      2f1e
dw      278d
dw      1fd4
dw      17fb
dw      100a
dw      0809
dw      0000
dw      f7f7
dw      eff6
dw      e805
dw      e02c
dw      d873
dw      d0e2
dw      c981
dw      c257
dw      bb6b
dw      b4c5
dw      ae6a
dw      a862
dw      a2b2
dw      9d61
dw      9873
dw      93ee
dw      8fd7
dw      8c30
dw      88ff
dw      8645
dw      8407
dw      8246
dw      8104
dw      8042
dw      8001
dw      8042
dw      8104
dw      8246
dw      8407
dw      8645
dw      88ff
dw      8c30
dw      8fd7
dw      93ee
dw      9873
dw      9d61
dw      a2b2
dw      a862
dw      ae6a
dw      b4c5
dw      bb6b
dw      c257
dw      c981
dw      d0e2
dw      d873
dw      e02c
dw      e805
dw      eff6
dw      f7f7
dw      0000
dw      0809
dw      100a
dw      17fb
dw      1fd4
dw      278d
dw      2f1e
dw      367f
dw      3da9
dw      4495
dw      4b3b
dw      5196
dw      579e
dw      5d4e
dw      629f
dw      678d
dw      6c12
dw      7029
dw      73d0
dw      7701
dw      79bb
dw      7bf9
dw      7dba
dw      7efc
dw      7fbe
dw      7fff
dw      7fbe
dw      7efc
dw      7dba
dw      7bf9
dw      79bb
dw      7701
dw      73d0
dw      7029
dw      6c12
dw      678d
dw      629f
dw      5d4e
dw      579e
dw      5196
dw      4b3b
dw      4495
dw      3da9
dw      367f
dw      2f1e
dw      278d
dw      1fd4
dw      17fb
dw      100a
dw      0809
dw      0000
dw      f7f7
dw      eff6
dw      e805
dw      e02c
dw      d873
dw      d0e2
dw      c981
dw      c257
dw      bb6b
dw      b4c5
dw      ae6a
dw      a862
dw      a2b2
dw      9d61
dw      9873
dw      93ee
dw      8fd7
dw      8c30
dw      88ff
dw      8645
dw      8407
dw      8246
dw      8104
dw      8042
dw      8001
dw      8042
dw      8104
dw      8246
dw      8407
dw      8645
dw      88ff
dw      8c30
dw      8fd7
dw      93ee
dw      9873
dw      9d61
dw      a2b2
dw      a862
dw      ae6a
dw      b4c5
dw      bb6b
dw      c257
dw      c981
dw      d0e2
dw      d873
dw      e02c
dw      e805
dw      eff6
dw      f7f7
dw	0000
    
