#---------------------------------------------------------------#
#	TITLE : IIR 4th filter(Canonic)				#
#	Ver : 0.0                                               #
#	Date : AUG. 3. 1995                                     #
#	Author : SunKyu-Kim                                     #
#	Program : IIR_4.sc                                      #
#---------------------------------------------------------------#
#  * Function :                                                 #
#        f0(n) = g0(n) = x(n)                                   #
#        fm(n) = fm(n-1) + km*gm-1(n-1)		                #
#        gm(n) = km*fm-1(n) + gm-1(m-1)                         #
#        y(n) = fN(n)						#
#                                                               #
#        f0(n)               f1(n)   fN-1(n)            fN(n)   #
#        ----------------->(+)-- - - --------------->(+)--->    #
#       |       k1 \   /                     \   /              #
# (n) --|            \                         \                #
#       |          /   \                     /    \     gN(n)   #
#        ---(z-1)--------->(+)-- - - --(z-1)-------->(+)--->    #
#        g0(n)                g1(n)  gN-1(n)			#
#                                                               #
#  * Input values						#
#	RAM1[10] : x[n]                                         #
#  * Parameters                                                 #
#	RAM1[20~2f] : g[n], f[n]                                #
#  * Coefficients                                               #
#	RAM0[20~2F] : k1, k2, ... , kN                          #
#  * Output values                                              #
#	RAM1[11~ ] : fN[n]                                      #
#                                                               #
#  * Cycle numbers                                              #
#	10-tap filter : maximum 139 cycles			#
#								#
#***************************************************************#

org     0000        # Interrupt vector table for SSP1601M

dw   	0004   # Reset address  #
dw   	0004   # INT0 address   #
dw  	0004   # INT1 address   #
dw   	0004   # INT2 address   #
org  	0004                    # for SSP1601S
   	
    lattic_z():
    # load reflection parameter k
        ldi	(r3|00), @k_table
        ldi	r0, 10
        ldi	r2, 40		# output pointer
        ld	a, st
        ori	a, 0200
        ld	st, a
        mod	f, setop
        ldi	a, 09
        ldi	(r3|01), be
    k_load_loop:
        ld      x, ((r3|00))
        ld      (r0+), x
        subi	1
        bra	n=0, @k_load_loop
    # 1 sample lattice zero filtering
        ldi	(r3|00), @input_dat  # start addr. of input_table
    1_samp_filtering:
        call	always, @lattic_z_main
        
#    host_wait:
#        call	usr0=1, @out_to_host
#        bra	always, @host_wait

    save_out:
        ld	a, (r3|01)
        subi	1
        ld	(r3|01), a
        bra	n=0, @1_samp_filtering
    lattice_end:
        bra	always, @lattice_end
        
        
        
#    out_to_host:
#        ld	ext0, a
#        ld 	-, stack
#        bra	always, @save_out
        
    lattic_z_main:
    # get input x(n)
        ldi	r4, 10 		# x[n]  or  fm-1[n]  pointer
        ld      x, ((r3|00))
        ld	(r4), x
    ## lattice filtering
    	ldi	r0, 10		# k pointer
        ldi	r5, 11		# g[n-1] pointer
    # get f1[n] and g0[n]
        mld	(r4), (r0)	# k1*f0[n]
        ld	a, (r5)		# g0[n-1]
        mpya	(r5), (r0)   # dummy for g1[n] = k1*f0[n] + g0[n-1]
        ld	(r3|11), a	# save g1[1] to temporary buffer
        mld	(r5), (r0+)	# k1*g0[n-1]
        ld	a, (r4) 	# f0[n] = x[n]
        ld	(r5), a		# save g0[n]
        mpya    (r5+), (r0)     # dummy for f1[n] = f0[n] + k1*g0[n-1]
        ld	(r4), a		# save f1[1] to temporary buffer
        
        
    # get f2[n] ~ f9[n],  g2[n] ~ g9[n]
        call	always, @cal_f_g   # f2[n], g2[n]
        call    always, @cal_f_g   # f3[n], g3[n]
        call    always, @cal_f_g   # f4[n], g4[n]
        call    always, @cal_f_g   # f5[n], g5[n]
        call    always, @cal_f_g   # f6[n], g6[n]
        call    always, @cal_f_g   # f7[n], g7[n]
        call    always, @cal_f_g   # f8[n], g8[n]
        call    always, @cal_f_g   # f9[n], g9[n]
    lattice_z_end:
        ld	(r2+), a
        ret
        
    cal_f_g:
	mld	(r5), (r0)	# km*gm-1[n-1]
	ld	a, (r4)		# fm-1[n]
        mpya    (r5), (r0)      # fm[n] = fm-1[n-1] + km*gm-1[n-1]
	ld	stack, a    	# save fm[n] temporarily
	
        mld     (r4), (r0+)     # km*fm-1[n]
        ld      a, (r5)         # gm-1[n-1]
        mpya    (r5), (r0)   # dummy for fm[n]= km*fm-1[n] + gm-1[n-1]
        ld	x, (r3|11)	# save gm-1[n] previosly calculated
        ld	(r5+), x
	ld	(r3|11), a  	# save gm[n] to buffer
	ld      (r4), stack	# save fm[n] to buffer
	ret
    k_table:
dw 	0ccc
dw	b334
dw	2666
dw	f334
dw	ccce
dw	1999
dw      2666
dw	c001
dw	0ccc
dw	ccce
    
    input_dat:
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000

org	fffc
dw	0004
dw	0004
dw	0004
dw	0004


