#------------------------------------------------------------#
#	Interpolator                                         #
#------------------------------------------------------------#
#  * Function :                                              #
#	M-rate Interpolator                                  #
#							     #
#           N-1                                              #
#    y[n] = SUM h[k]*x[(n-k)/M]                              #
#	    k=0                                              #
#                                                            #
#  * Assumption and Constraints				     #
#							     #
#  * Input Values                                            #
# 	RAM0[10~2F] : X[n] buffer                            #
#       RAM1[10~10+N] : h[k]                                 #
#  * Temporary Variables				     #
#	RAM0[00] table address                               #
#	RAM0[01] loop size selector			     #
#	RAM0[10] output counter                              #
#  * Output Values                                           #
#	RAM0[40~FF] : y[n] buffer                            #
#  * Number of cycles                                        #
#       8 tap filter : 50 cycles                             #
#------------------------------------------------------------#
# user definable value
h_size_N:     		equ 	008
interp_factor_M:	equ	003
interp_factor_Msub1:	equ	002
output_counter_number:	equ	028

	org	0000
	dw	0004
	dw	0004
	dw	0004
	dw	0004
	org	0004
## Initial Processing
     # set N selector
    	ldi	a, @h_size_N
    	subi	1
        cmpi	a, 32
        bra	n=0, @loopsize_64or128
        cmpi	a, 16
        bra	n=0, @loopsize_32
        cmpi	a, 8
        bra	n=0, @loopsize_16
        ldi	(r3|01), 00	# loopsize=8
        bra	always, @load_h[k]
     loopsize_16:
        ldi     (r3|01), 04     # loopsize=16
        bra	always, @load_h[k]
     loopsize_32:
        ldi	(r3|01), 08	# loopsize=32
        bra	always, @load_h[k]
     loopsize_64or128:
 	cmpi	a, 40
 	bra	n=0, @loopsize_128
        ldi     (r3|01), 0c     # loopsize=64
        bra	always, @load_h[k]
     loopsize_128:
        ldi	(r3|01), 0f	# loopsize=128
     # If (0 < N <= 8)  ->  N selector=00
     # If (8 < N <= 16) ->  N selector=04
     # If (16 < N <= 32)  ->  N selector=08
     # If (32 < N <= 64)  ->  N selector=0c
     # If (64 < N <= 128)  ->  N selector=0f
     # load filter coefficients h[k]
     load_h[k]:
    	ldi	r4, 10
    	ldi	(r3|00), @h_table
     # N is changed to power of 2 for circular buffer
        ld	a, (r3|01)	# load N selector
        addi	a, @set_coeff_size
        ld	pc, a
        
     set_coeff_size:
        ldi	a, 08		# counter : coeff. table size (N=8)
        bra	always, @load_coeff
        ldi	a, 10		# counter : coeff. table size (N=16)
        bra	always, @load_coeff
        ldi	a, 20		# counter : coeff. table size (N=32)
        bra	always, @load_coeff
        ldi	a, 40		# counter : coeff. table size (N=64)
        bra	always, @load_coeff
        ldi	a, 80		# counter : coeff. table size (N=64)
     load_coeff:
        ld	stack, ((r3|00)) # load coeff. from P-Memory, (r3|00)++
        ld	(r4+), stack
        subi	1
        bra	z=0, @load_coeff
     # set loop size
        ld	a, (r3|01)
        addi    a, @set_loop_size
        ld	pc, a
     set_loop_size:
        ld	a, st
        ori	03		# loop size = 8  (N=8)
        bra	always, @save_st
        ld	a, st
        ori	04		# loop size = 16 (N=16)
        bra	always, @save_st
        ld	a, st
        ori	05		# loop size = 32 (N=32)
        bra	always, @save_st
        ld	a, st
        ori     06	        # loop size = 64 (N=64)
        bra	always, @save_st
        ld	a, st
        ori     07      	# loop size = 128 (N=128)
    save_st:
    	ld	st, a
    	
## Interpolate block
    # init some value
        ldi	r0, 10          # input pointer
        ldi	r4, 10		# filter coefficients pointer
        ldi	r1, 40		# output pointer
        ldi	(r3|10), @output_counter_number
        ldi	(r3|00), @input_table
        ldi	(r3|11), 00
        mod	f, setop
    interp_main:
        ldi	(r3|01), @interp_factor_Msub1
    cal_M_sample:
    # load 1 input and 2 zero extention
    	ld	x, ((r3|00))
    	ld      (r0), x
    	call	always, @filtering
    cal_interp_sample:
    	ld	a, (r3|01)
    	subi	1
    	ld	(r3|01), a
    	bra     n=1, @count_output
    	ldi	(r0), 00
    	call	always, @filtering
	bra	always, @cal_interp_sample
    count_output:               # decrease output counter
    	ld	a, (r3|10)
    	subi	1
    	ld	(r3|10), a
    	bra	n=0, @interp_main
    # interpolator end
    end:
    	bra	always, @end

    filtering:
    	mld	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0-)
    	mpya	(r4+), (r0)
    	add	a, p
    	mod	always, shr
# for test
    	ld	y, a
    	ld	x, st
    	ld	a, st
    	andi	a, fff8
    	ld	st, a
    	ld	(r1+), y
    	ld	st, x
# test end
    	ret
    	
    	
    h_table:
dw 	ffc9
dw	0418
dw	de8b
dw	205a
dw	205a
dw	de8b
dw      0418
dw	ffc9
    
    input_table:
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000

org	fffc
dw	0004
dw	0004
dw	0004
dw	0004


     
     
    	
        

    	

	

