#-----------------------------------------------------------
#	U_law Encoder and Decoder
#-----------------------------------------------------------
#  * Function
#    compress 13 bit linear data to 8 bit u_law data
#		   ln(1+u|x|/xmax)
#	y(x)= xmax --------------- sgn(x)
#	            ln(1+u)
#  * Assumption and Constraint
#
#  * Input Value
#	RAM0[10] : input 13 bit linear data
#  * Temporary Values
#	RAM0[00] : segment code
#	RAM0[01] : segment base value
#	RAM0[11] : shift value to get interval number(0~15)
#	RAM1[00] : sign
#  * Output Value
#	RAM0[10] : encoder output 8 bit
#  * Cycles
#	encoder : 40 cycles, decoder : 48 cycles
#-------------------------------------------------------------------
org	0000
dw      0004
dw      0004
dw      0004
dw      0004
org	0004
    ## u_law encoder
    # Input 13 bit linear data
    	ld	a, (r3|10)	# 16 bit linear data
    	mod	always, shr
    	mod	always, shr	# 13 bit linear data
    	cmpi	00
    	ldi	(r7|00), 0080	# sign bit = 1 when pos
    	bra	n=0, @pos_operat
    neg_operat:
    	mod	always, abs
        ldi     (r7|00), 0000   # sign bit = 0 when neg
    pos_operat:
    	cmpi	a, 200		# cmp with 512
	bra	n=1, @lower_than_512
	cmpi	a, 3df
	bra	n=1, @fix_seg_5	# cmp with 1024
	cmpi	a, 7df
	bra	n=1, @fix_seg_6
	cmpi	a, fdf
	bra	n=1, @fix_seg_7
	ldi	(r3|00), 00	# fix segment 8
	ldi	(r3|01), fdf
	ldi	(r3|11), 0080
	bra	always, @fix_seg_end
    fix_seg_7:
	ldi	(r3|00), 10	# fix segment 7
	ldi	(r3|01), 7df
	ldi	(r3|11), 0100
	bra	always, @fix_seg_end
    fix_seg_5:
        ldi     (r3|00), 30     # fix segment 5
	ldi	(r3|01), 1df
	ldi	(r3|11), 0400
	bra	always, @fix_seg_end
    fix_seg_6:
        ldi     (r3|00), 20    # fix segment 6
	ldi	(r3|01), 3df
	ldi	(r3|11), 0200
	bra	always, @fix_seg_end
    lower_than_512:
     	cmpi	5f
     	bra	n=1, @lower_than_5f
     	cmpi	df
     	bra	n=1, @fix_seg_3
     	ldi	(r3|00), 40	# fix segment 4
	ldi	(r3|01), df
	ldi	(r3|11), 0800
	bra	always, @fix_seg_end
    fix_seg_3:
        ldi     (r3|00), 50     # fix segment 3
	ldi	(r3|01), 5f
	ldi	(r3|11), 1000
	bra	always, @fix_seg_end
    lower_than_5f:
        cmpi	1f
        bra	n=1, @fix_seg_1
        ldi     (r3|00), 60     # fix segment 2
	ldi	(r3|01), 1f
	ldi	(r3|11), 2000
	bra	always, @fix_seg_end
    fix_seg_1:
        cmpi	0
        bra	z=1, @fix_seg_0
        
        ldi     (r3|00), 70     # fix segment 1
	ldi	(r3|01), 00
	ldi	(r3|11), 4000
	bra	always, @fix_seg_end
    fix_seg_0:
       ldi	a, 007f		# segment + interval number
       bra	always, @combine_sign
    fix_seg_end:
        sub 	a, (r3|01)	# sub segment start value
	ld	x, a
	ld	y, (r3|11)	# shift value
	ld	a, p		# get interval number
	eori	a, 0f		# toggle bits
	or 	a, (r3|00)	# combine segment & intervals
    combine_sign:
	or	a, (r7|00)	# combine sign
	ld	(r3|10), a  	# save u_law 8 bit code
    	
    ## u_law decoder
    # input u_law code
    	ld	a, (r3|10)	# load u_law 8 bit code
    	andi	a, 0070		# get segment
    	cmpi	40
    	bra	n=0, @lower_than_seg4
    	cmpi	20
    	bra	n=0, @lower_than_seg6
    	cmpi	10
    	bra	n=0, @fix_base_7df
    	ldi	(r3|01), fdf       	# seg=8
    	ldi	(r3|11), 100		# interval size=100
    	bra	always, @fix_base_end
    fix_base_7df:
    	ldi	(r3|01), 7df       	# seg=7
    	ldi	(r3|11), 80		# interval size=80
    	bra	always, @fix_base_end
    lower_than_seg6:
    	cmpi	30
    	bra	n=0, @fix_base_1df
    	ldi	(r3|01), 3df       	# seg=6
    	ldi	(r3|11), 40		# interval size=40
    	bra	always, @fix_base_end
    fix_base_1df:
    	ldi	(r3|01), 1df       	# seg=5
    	ldi	(r3|11), 20		# interval size=20
    	bra	always, @fix_base_end
    lower_than_seg4:
	cmpi	60
	bra	n=0, @lower_than_2
	cmpi	50
	bra	n=0, @fix_base_5f
    	ldi	(r3|01), df       	# seg=4
    	ldi	(r3|11), 10		# interval size=10
    	bra	always, @fix_base_end
    fix_base_5f:
    	ldi	(r3|01), 5f    		# seg=3
    	ldi	(r3|11), 8		# interval size=8
    	bra	always, @fix_base_end
    lower_than_2:
    	cmpi	70
    	bra	n=0, @fix_base_0
    	ldi	(r3|01), 1f    		# seg=2
    	ldi	(r3|11), 4		# interval size=4
    	bra	always, @fix_base_end
    fix_base_0:
    	ld	a, (r3|10)
    	cmpi	ff
    	bra	z=1, @zero_case
    	ldi	(r3|01), 00    		# seg=1
    	ldi	(r3|11), 2		# interval size=2
    	bra	always, @fix_base_end
    zero_case:
        ldi 	a, 00
        bra	always, @save_pcm
    fix_base_end:
        ld	a, (r3|10)      # 8 bit u_law code
        andi	0f		# get interval numbers
        eori	0f		# toggle bits
        ld      x, a            # for interval number * interval size
        bra	z=0, @non_zero_interval
    non_zero_interval:
        ld	y, (r3|11)	# interval size
        ld	a, p            # interval number * interval size
        mod	always, shr	# compensate double in mac oper.
        ld	stack, ext7
        ld	a, (r3|01)
        add	a, stack  	# x(n-1)= seg. base value + interval value
        ld	stack, a        # save x(n-1) to stack for y(n)
        add	a, (r3|11)      # get x(n)
	add	a, stack
	mod	always, shr     # y(n) = ( x(n) + x(n-1) ) / 2
	ld	stack, a	# save y(n) to stack for sign consideration
	ld	a, (r3|10)
	andi	a, 80           # get sign info.  a=80  -> sign=pos
	ld	a, stack
	bra	z=0, @save_pcm
	mod	always, neg     # if sign=neg  then y(n)=-y(n)
    save_pcm:
    	ld	(r3|10), a	# save y(n)
    decoder_end:
    	bra	always, @decoder_end
	
    	
        
     
